/**********************************************************************
 *                                                                    *
 *  cfKolabAdmin - QT application for simple user administration of   *
 *    a Kolab1 server.                                                *
 *                                                                    *
 *  Copyright (c) 2004  Code Fusion cc                                *
 *  Writen by                                                         *
 *    Stephan Buys  <s.buys@codefusion.co.za>                         *
 *    Stuart Bing  <s.binge@codefusion.co.za>                        *
 *                                                                    *
 *  This  program is free  software; you can redistribute  it and/or  *
 *  modify it  under the terms of the GNU  General Public License as  *
 *  published by the  Free Software Foundation; either version 2, or  *
 *  (at your option) any later version.                               *
 *                                                                    *
 *  This program is  distributed in the hope that it will be useful,  *
 *  but WITHOUT  ANY WARRANTY; without even the  implied warranty of  *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  *
 *  General Public License for more details.                          *
 *                                                                    *
 *  You can view the  GNU General Public License, online, at the GNU  *
 *  Project's homepage; see <http://www.gnu.org/licenses/gpl.html>.   *
 *                                                                    *
 **********************************************************************/

#include <utility>
#include <strings.h>

#include <qpainter.h>
#include <qfont.h>

#include "LdapListViewItem.h"
#include "LdapAttributes.h"

LdapListViewItem::LdapListViewItem(QListView * parent, LDAP * ldap_conn, LDAPMessage * ldap_entry) :
    QListViewItem(parent),
    has_loaded_attrs(false),
    awaiting_removal(false),
    ldap_connection(ldap_conn)
{
    // Check if this object is awaiting removal
    char ** values = ldap_get_values(ldap_connection, ldap_entry, "deleteflag");
    if (ldap_count_values(values))
        awaiting_removal = strcasecmp(*values, "true") == 0;
    ldap_value_free(values);

    // Store the DN of this object
    char * text = ldap_get_dn(ldap_connection, ldap_entry);
    entry_dn = text;
    ldap_memfree(text);
}

void LdapListViewItem::paintCell(QPainter * p, const QColorGroup & cg,
                                 int column, int width, int alignment)
{
    QColorGroup _cg(cg);
    QColor c = _cg.text();

    if (awaiting_removal) {
        _cg.setColor(QColorGroup::HighlightedText, Qt::gray);
        _cg.setColor(QColorGroup::Foreground, Qt::gray);
        _cg.setColor(QColorGroup::Text, Qt::gray);
    }

    QListViewItem::paintCell(p, _cg, column, width, alignment);

    _cg.setColor(QColorGroup::Text, c);

/*    QFont f = p->font();

    if (awaiting_removal) {
        QFont removed(f);
        removed.setStrikeOut(true);
        p->setFont(removed);
    }

    QListViewItem::paintCell(p, cg, column, width, alignment);

    p->setFont(f);*/
}

void LdapListViewItem::paintFocus(QPainter * p, const QColorGroup & cg,
                                  const QRect & r)
{
    QColorGroup _cg(cg);
    QColor c = _cg.text();

    if (awaiting_removal) {
        _cg.setColor(QColorGroup::HighlightedText, Qt::gray);
        _cg.setColor(QColorGroup::Foreground, Qt::gray);
        _cg.setColor(QColorGroup::Text, Qt::gray);
    }

    QListViewItem::paintFocus(p, _cg, r);

    _cg.setColor(QColorGroup::Text, c);

/*    QFont f = p->font();

    if (awaiting_removal) {
        QFont removed(f);
        removed.setStrikeOut(true);
        p->setFont(removed);
    }

    QListViewItem::paintFocus(p, cg, r);

    p->setFont(f);*/
}

QString LdapListViewItem::reload(bool * dne)
{
    // Reload the attributes of the current LDAP object
    clearAttrs();
    return getAttrs(dne);
}

QString LdapListViewItem::remove()
{
    // Remove the current LDAP object by setting deleteflag
    LdapAttrVec attrs;
    initLDAPAttributes(attrs);
    addLDAPAttributes(attrs, "deleteflag", "TRUE", LDAP_MOD_ADD);

    int result = ldap_modify_s(ldap_connection, entry_dn.ascii(), &attrs[0]);

    clearLDAPAttributes(attrs);

    if (result == LDAP_SUCCESS) {
        awaiting_removal = true;
        return QString::null;
    }

    return ldap_err2string(result);
}

QString LdapListViewItem::nuke()
{
    // Nuke the current object out of LDAP
    int result = ldap_delete_s(ldap_connection, entry_dn.ascii());
    if (result == LDAP_SUCCESS) return QString::null;

    return ldap_err2string(result);
}

QString LdapListViewItem::getAttrValue(const char * attr)
{
    // Get the value of the specified attribute (only supports single values at the moment)
    AttrValueIter i = attr_value_map.find(attr);
    return (i == attr_value_map.end() ? QString::null : QString(*(i->second)));
}

QString LdapListViewItem::loadAttrs(char ** attrs, bool * dne)
{
    // Load the specified attribute list
    if (has_loaded_attrs) return QString::null;

    // Set up our logging variables
    QString log_text;
    QTextOStream log(&log_text);

    LDAPMessage * ldap_message = 0;

    // Load the requested attributes
    int result = ldap_search_s(
        ldap_connection,
        entry_dn.ascii(),
        LDAP_SCOPE_BASE,
        "(objectClass=*)",
        attrs,
        0,
        &ldap_message
    );

    if (result == LDAP_NO_SUCH_OBJECT && dne) {
        *dne = true;
        return QString::null;
    } else if (result != LDAP_OPT_SUCCESS) {
        log_text = "";
        log
            << "<font color=\"#FF0000\">Unable to load attributes from DN "
            << entry_dn
            << "</font>";
        return log_text;
    }

    if (!ldap_count_entries(ldap_connection, ldap_message)) {
        log_text = "";
        log
            << "<font color=\"#FF0000\">No attributes found for DN "
            << entry_dn
            << "</font>";
        return log_text;
    }

    // Add the (attribute => values) pair to the attribute value map
    LDAPMessage * ldap_entry = ldap_first_entry(ldap_connection, ldap_message);

    BerElement * ber_element = 0;
    char * attribute = ldap_first_attribute(ldap_connection, ldap_entry, &ber_element);
    do {
        char ** values = ldap_get_values(ldap_connection, ldap_entry, attribute);
        if (!strcasecmp(attribute, "deleteflag"))
            awaiting_removal = strcasecmp(*values, "true") == 0;
        attr_value_map.insert(std::make_pair(attribute, values));
        ldap_memfree(attribute);
    } while ((attribute = ldap_next_attribute(ldap_connection, ldap_entry, ber_element)));

    ber_free(ber_element, 0);

    has_loaded_attrs = true;

    return QString::null;
}

void LdapListViewItem::clearAttrs()
{
    // Free all our attribute values
    for (AttrValueIter i = attr_value_map.begin(); i != attr_value_map.end(); ++i)
        ldap_value_free(i->second);

    attr_value_map.clear();

    has_loaded_attrs = false;
}

void LdapListViewItem::populateListView(QListView * list_view)
{
    // Insert the loaded attributes & their values into the given list view, as key/value pairs
    // Multiple values per attribute are added as seperate entries within the list view
    for (AttrValueIter i = attr_value_map.begin(); i != attr_value_map.end(); ++i) {
        char ** value = i->second;
        int attr_count = ldap_count_values(value);
        for (int count = 0; count < attr_count; ++count)
            new QListViewItem(list_view, getAttrDesc(i->first), *(value++));
    }
}
